/*
	Copyright (c) 2006-2010, ePages GmbH
	All Rights Reserved.

	epages.widget.Uploader $Revision: 1.5 $
*/

dojo.provide("epages.widget.Uploader");

dojo.require("dojo.fx");
dojo.require("dojo.fx.easing");
dojo.require("epages.fileTransfer");
dojo.require("epages.browser");
dojo.require("epages.widget");
dojo.require("epages.widget.Modaldialog");
dojo.require("epages.widget.FormElements");
dojo.require("epages.templated");
dojo.require("epages.io.translation");

(function(dj){

	// translate
	
	var dict = epages.io.dictionary.translation,
	
		translate = function( key ){
			return dict[key] || '{'+key+'}';
		},
	
	// cache handle for files to upload
		Cache = function( hash ) {

			this.length = 0;
			this.index = -1;
			this.pointer = -1;

			if( hash ) {
				for( var i in hash ) {
					this.add( this[i] )
				}
			}
		};

		dj.mixin( Cache.prototype, {
			add: function( value ) {
				if( value!==undefined ) {
					this.length++;
					this.index++;
					this[this.index] = value;
				}
			},

			get: function(i) {
				if(i===undefined) {
					return this[this.pointer];
				}
				else {
					return this[i];
				}
			},

			remove: function(i) {
				if( this[i] ) {
					this.length -= 1;
					delete( this[i] );
				}
			},

			prev: function() {
				this.pointer -= 1;

				for( ; this.pointer > 0 ;  ) {
					if( this[this.pointer] === undefined  ) {
						this.pointer -= 1;
					}
					else {
						break;
					}
				}
			},

			next: function() {
				this.pointer++;

				for( ; this.pointer < this.index ;  ) {
					if( this[this.pointer] === undefined  ) {
						this.pointer++;
					}
					else {
						break;
					}
				}
			},

			each: function( fn, callback ) {

				for( var i=0; i <= this.index ; i++ ) {
					if( this[i] !== undefined  ) {
						fn( this[i], i )
					}
				}

				if( callback ) {
					callback();
				}
			},

			reset: function(i) {
				this.pointer = -1;
			}
		});





	dj.declare("epages.widget.Uploader", [ epages.Templated, epages.widget.Modaldialog ], {

		_uploadHandle:				null,
		_fileList:					null,
		_inputFileConnect:			null,
		_sendInAction:				false,

		templates: {
			'uploadList':			'<table style="width:100%;table-layout:fixed;">\
										<tbody dojoAttachpoint="uploadList">\
										</tbody>\
									</table>',
			'uploadListItem':		'<table>\
										<tr style="height:35px;" dojoAttachpoint="domNode">\
											<td style="width:40px;align:center;vertical-align:middle;">\
												<span dojoAttachpoint="itemIcon"></span>\
											</td>\
											<td>\
												<div dojoAttachpoint="itemTitle" style="color:#666;font-size:10px;white-space:nowrap;overflow:hidden;"></div>\
												<div class="StatusBar" style="width:100%; margin:2px; float:none;">\
													<div class="InnerStatusBar">\
														<div dojoAttachpoint="itemStatus" class="PercentageNormal"></div>\
													</div>\
												</div>\
												<div dojoAttachpoint="itemComment" style="color:#666;font-size:10px;white-space:nowrap;overflow:hidden;"></div>\
											</td>\
											<td style="width:30px;align:center;vertical-align:middle;">\
												<a dojoAttachpoint="itemRemove" href="javascript:;" class="Sprite ico_s_remove"></a>\
											</td>\
										</tr>\
									<table>'
		},

		postMixInProperties: function() {
			this.inherited('postMixInProperties',arguments);

			this.options.accept =		this.options.accept || '*/*',
			this.options.sendaction = 	this.options.sendaction || '?',
			this.options.deleteaction =	this.options.deleteaction || false,
			this.options.finishaction =	this.options.finishaction || '',
			this.options.name =			this.options.name || 'NewFile'

			this._uploadHandle = new epages.FileTransfer({
				accept:			this.options.accept,
				sendaction: 	this.options.sendaction,
				deleteaction:	this.options.deleteaction,
				name:			this.options.name
			});

			this.options.accept = this._uploadHandle.options.accept;

			this._fileList = new Cache();

		},

		postCreate: function() {
			this.inherited('postCreate',arguments);

			this.setupContent({
				width: 			350,
				title:			'Uploader'
			});
			// show the button bar
			dj.style( this.innerButtonBarNode, {'display':''});
			// create uploadlist by getting template
			var tmpl = this.getTemplate('uploadList');
			dj.place( tmpl.domNode, this.containerNode, 'append' );
			this.uploadListNode = tmpl.uploadList;
			dj.style( this.containerNode, { // TODO: css class for styling
				'min-height':	'90px',
				'max-height':	'255px',
				'overflow':		'auto',
				'padding':		'0'
			});
			// add formNode
			var form = dj.create('form',{name:this.id+'_buttonForm'},this.innerButtonBarNode,'append');
			// create addButton widget
			this.addButton = new epages.widget.FormFile( {}, dj.create('input',{type:'file'},form,'append') )
				.attr({
					'accept':   this.options.accept || '',
					'multiple': this.options.multiple || false
				})
				.addHandle( 'domNode.change.Uploader', 'domNode', 'change', dj.hitch(this,this._addFiles) )
				.connectHandle( 'domNode.change.Uploader' );
			dj.query( 'span', this.addButton.wrapNode)[0].innerHTML = '+';
			dj.addClass( this.addButton.wrapNode, 'FloatLeft' );
			// create finishButton widget
			this.finishButton = new epages.widget.FormButton( {}, dj.create('input',{type:'button'},form,'append') )
				.attr({
					'value': translate('finish') || '{finish}'
				})
				.addHandle( 'domNode.click.Uploader', 'domNode', 'click', dj.hitch(this,this._finishButtonAction) )
				.connectHandle( 'domNode.click.Uploader' );
			// create dndInput
			this.dndButton = new epages.widget.FormFile( {}, dj.create('input',{type:'file'},form,'append') )
				.attr({
					'accept':   this.options.accept || '',
					'multiple': this.options.multiple || false
				})
				.style( 'domNode',{
					'position':  'absolute',
					'top':       0,
					'right':     0,
					'bottom':    0,
					'left':      0,
					'margin':    0
				})
				.style( 'wrapNode',{
					'position':  'absolute',
					'display':   'none',
					'top':       0,
					'right':     0,
					'bottom':    0,
					'left':      0,
					'margin':    0,
					'opacity':   0
				})
				.addHandle( 'domNode.change.Uploader', 'domNode', 'change', dj.hitch(this,this._addFiles) )
				.connectHandle( 'domNode.change.Uploader' )
			if( !epages.Browser.engine=='MSIE' ){
				this.dndButton
					.addHandle( 'window.focus.Uploader',   window,    'focus', function(){ dj.style(this.wrapNode,{'display':'none'}); })
					.addHandle( 'window.blur.Uploader',    window,    'blur', function(){ dj.style(this.wrapNode,{'display':'block'}); })
					.connectHandle( 'window.focus.Uploader' )
					.connectHandle( 'window.blur.Uploader' );
			}

			this.show();
		},

		externalInput: function( node ) {
			this._addFiles({target:node});
		},

		_addFiles: function(event) {
			if( event.target.files ) {
				var files = event.target.files;
				for( var i=0,iLength=files.length ; i<iLength ; i++ ) {
					this._fileList.add({
						nodes:		this.getTemplate('uploadListItem'),
						data:		{ uploaded: false },
						file:		this._uploadHandle.checkFile( files[i] )
					});
					this._addUploadListItem(this._fileList.index);
				}
			}
			else {
				this._fileList.add({
					nodes:		this.getTemplate('uploadListItem'),
					data:		{ uploaded: false },
					file:		this._uploadHandle.checkFile( event.target )
				});

				this._addUploadListItem(this._fileList.index);
			}

			var inputDisbale = this.options.multiple!=true && this._fileList.length >=1 ? true : false;

			this.addButton.attr({
					'value':    '',
					'disabled': inputDisbale
				});
			this.dndButton.attr({
					'value':    '',
					'disabled': inputDisbale
				});
			this.finishButton.attr({
					'disabled': true
				});

			if( !this._sendInAction ) {
				this._sendInAction = true;
				this._fileList.next();
				this._sendUploadListItem( this._fileList.get(), this._fileList.pointer );
			}
		},

		_addUploadListItem: function(i) {
			var listItem = this._fileList.get(i);
			dj.place( listItem.nodes.domNode, this.uploadListNode, 'append' );

			dj.addClass( listItem.nodes.itemIcon, 'Sprite '+listItem.file.icon );
			listItem.nodes.itemTitle.innerHTML = listItem.file.name;
			dj.style( listItem.nodes.itemStatus, {'width':'0'} );

			listItem.data.removeConnect = dj.connect( listItem.nodes.itemRemove, 'click', this, function() { this._removeUploadListItem(i); } );

			this._zebraUploadList();

			this.layout();
		},

		_removeUploadListItem: function(i) {
			var listItem = this._fileList.get(i);

			if( listItem.data.uploaded ){
				// TODO: params to send a deleteaction
				this._uploadeHandle.deleteFile( {} );
			}

			dj.disconnect( listItem.data.removeConnect );

			listItem.nodes.domNode.parentNode.removeChild( listItem.nodes.domNode );

			this._fileList.remove(i);

			this._zebraUploadList();

			if( this._fileList.length<=0 ) {
				this.finishButton.attr('disabled',true);
			}

			if( this.options.multiple!==true && this._fileList.length < 1 ) {
				this.addButton.attr('disabled',false);
			}

			this.layout();
		},

		_sendUploadListItem: function( listItem, i ) {
			// hide remove button if no delete action
			listItem = this._fileList.get(i);
			if( listItem ){
				if( !this.options.deleteaction ) {
					dj.style( listItem.nodes.itemRemove, {'display':'none'} );
				}

				// animate statusbar
				listItem.data.anim = dj.fx.combine([
					dj.animateProperty({
						node: 		listItem.nodes.itemStatus,
						duration: 	this._uploadHandle.getFileSendTime(listItem.file),
						easing: 	dojo.fx.easing.quadIn,
						properties: {
							width:{
								start: 0,
								end: listItem.nodes.itemStatus.parentNode.offsetWidth - 2,
								unit:'px'
							}
						}
					})
				]);
				listItem.data.anim.play();

				// scroll to this item
				var top = listItem.nodes.domNode.offsetTop,
					middle = (this.containerNode.offsetHeight/2)+listItem.nodes.domNode.offsetHeight,
					diff = top-middle;

				this.containerNode.scrollTop = diff>top ? top : diff;

				// send with upload handle
				this._uploadHandle.sendFile(
					listItem.file,
					dj.hitch(this,function(response){
						this._finishUploadListItem( response, i );
					})
				);
			}
			else{
				this._nextUploadListItem();
			}
		},
		_nextUploadListItem: function(){
			this._fileList.next();
			if( this._fileList.pointer > this._fileList.index ) {
				
				this._fileList.each(dj.hitch(this,function( listItem ){
					if( listItem.data.error ){
						dj.place( listItem.nodes.domNode, this.uploadListNode, 'last' );
					}
				}));
				
				this._fileList.prev();
				this._sendInAction = false;
				this.finishButton.attr('disabled',false);
			}
			else {
				this._sendUploadListItem( this._fileList.get(), this._fileList.pointer );
			}
		},

		_finishUploadListItem: function( response, i ) {
			this._nextUploadListItem();

			var listItem = this._fileList.get(i);

			listItem.data.anim.stop();
			dj.style( listItem.nodes.itemStatus, {width: (listItem.nodes.itemStatus.parentNode.offsetWidth - 2)+'px'} );

			if( response.Errors ) {
				listItem.data.error = true;
				dj.addClass( listItem.nodes.itemStatus, 'PercentageCritical' );
				listItem.nodes.itemComment.innerHTML = translate(response.Errors[0].Reason);
				dj.style( listItem.nodes.itemRemove, {'display':''} );
			}
			else {
				listItem.data.uploaded = true;
				listItem.nodes.itemComment.innerHTML = translate('UploadSuccess') +' '+ i;

				if( this.options.deleteaction ) {
					dj.style( listItem.nodes.itemRemove, {'display':''} );
					dj.removeClass( listItem.nodes.itemRemove, 'ico_s_remove' );
					dj.addClass( listItem.nodes.itemRemove, 'ico_s_delete' );
				}
			}
		},

		_zebraUploadList: function() {
			var list = this.uploadListNode.childNodes;

			for( var i=0,iLength=list.length ; i<iLength ; i++ ) {
				if( i % 2 ) {
					dj.addClass( list[i], 'alternate' );
				}
				else {
					dj.removeClass( list[i], 'alternate' );
				}
			}
		},

		_finishButtonAction: function() {
			this.hide();
			if( typeof this.options.finishaction === 'function' ){
				this.options.finishaction();
			}
			else if( this.options.finishaction=='reload' ) {
				location.href = epages.vars.BaseUrl + '?ViewAction=' + epages.vars.ViewAction + '&ObjectID=' + epages.vars.ObjectID;
			}
		}

	});

})(dojo);